@@ -942,22 +942,11 @@ PHP_APCU_API zend_bool apc_cache_insert(apc_cache_t* cache,
                                         zend_bool exclusive)
 {
 	zend_bool result = 0;
-	zend_bool bailout = 0;
 
-    APC_LOCK(cache->header);
-
-	zend_try {
+	php_apc_try(APC_LOCK(cache->header), {
 		result = apc_cache_insert_internal(
 			cache, key, value, ctxt, t, exclusive);
-	} zend_catch {
-		bailout = 1;
-	} zend_end_try();
-
-	APC_UNLOCK(cache->header);
-
-	if (bailout) {
-		zend_bailout();
-	}
+	}, APC_UNLOCK(cache->header));
 
 	return result;	
 }
@@ -1066,7 +1055,7 @@ PHP_APCU_API zend_bool apc_cache_update(apc_cache_t* cache, zend_string *key, ap
     apc_cache_slot_t** slot;
     apc_cache_entry_t tmp_entry;
 	
-    zend_bool retval = 0, bailout = 0;
+    zend_bool retval = 0;
     zend_ulong h, s;
 
     if(apc_cache_busy(cache))
@@ -1078,10 +1067,8 @@ PHP_APCU_API zend_bool apc_cache_update(apc_cache_t* cache, zend_string *key, ap
     /* calculate hash */
     apc_cache_hash_slot(cache, key, &h, &s);
 	
-	/* lock header */
-	APC_LOCK(cache->header);
+	php_apc_try(APC_LOCK(cache->header), {
 
-	zend_try {
 		/* find head */
 		slot = &cache->slots[s];
 
@@ -1111,25 +1098,17 @@ PHP_APCU_API zend_bool apc_cache_update(apc_cache_t* cache, zend_string *key, ap
 		            }
 		            break;
 		        }
-				/* unlock header */
-				APC_UNLOCK(cache->header);
 
-		        return retval;
+				php_apc_try_end({
+					APC_UNLOCK(cache->header);
+					return retval;
+				});
 		    }
 
 			/* set next slot */
 		    slot = &(*slot)->next;
 		}
-	} zend_catch {
-		bailout = 1;
-	} zend_end_try();
-
-	/* unlock header */
-	APC_UNLOCK(cache->header);
-
-	if (bailout) {
-		zend_bailout();
-	}
+	}, APC_UNLOCK(cache->header));
 
 	/* failed to find matching entry, create it */
 	ZVAL_LONG(&tmp_entry.val, 0);
@@ -1730,7 +1709,6 @@ PHP_APCU_API zval apc_cache_info(apc_cache_t* cache, zend_bool limited)
     zval slots;
     apc_cache_slot_t* p;
     zend_ulong i, j;
-	zend_bool bailout = 0;
 
     if (!cache) {
 		ZVAL_NULL(&info);
@@ -1738,10 +1716,7 @@ PHP_APCU_API zval apc_cache_info(apc_cache_t* cache, zend_bool limited)
         return info;
     }
 
-    /* read lock header */
-    APC_RLOCK(cache->header);
-
-	zend_try {
+	php_apc_try(APC_RLOCK(cache->header), {
 
 		array_init(&info);
 		add_assoc_long(&info, "num_slots", cache->nslots);
@@ -1791,16 +1766,7 @@ PHP_APCU_API zval apc_cache_info(apc_cache_t* cache, zend_bool limited)
 		    add_assoc_zval(&info, "slot_distribution", &slots);
 		}
 
-	} zend_catch {
-		bailout = 1;
-	} zend_end_try();
-
-	/* unlock header */
-	APC_RUNLOCK(cache->header);
-
-	if (bailout) {
-		zend_bailout();
-	}
+	}, APC_RUNLOCK(cache->header));
 
     return info;
 }
@@ -1812,15 +1778,11 @@ PHP_APCU_API zval apc_cache_info(apc_cache_t* cache, zend_bool limited)
 PHP_APCU_API zval* apc_cache_stat(apc_cache_t* cache, zend_string *key, zval *stat) {
     apc_cache_slot_t** slot;
 	zend_ulong h, s;
-    zend_bool bailout = 0;
 
 	/* calculate hash and slot */
 	apc_cache_hash_slot(cache, key, &h, &s);
 
-    /* read lock header */
-	APC_RLOCK(cache->header);
-
-	zend_try {
+	php_apc_try(APC_RLOCK(cache->header), {
 		/* find head */
 		slot = &cache->slots[s];
 
@@ -1844,15 +1806,7 @@ PHP_APCU_API zval* apc_cache_stat(apc_cache_t* cache, zend_string *key, zval *st
 			/* next */
 			slot = &(*slot)->next;		
 		}
-	} zend_catch {
-		bailout = 1;
-	} zend_end_try();
-
-    APC_RUNLOCK(cache->header);
-
-	if (bailout) {
-		zend_bailout();
-	}
+	}, APC_RUNLOCK(cache->header));
 
     return stat;
 }
@@ -1922,7 +1876,6 @@ PHP_APCU_API void apc_cache_serializer(apc_cache_t* cache, const char* name) {
 
 PHP_APCU_API void apc_cache_entry(apc_cache_t *cache, zval *key, zend_fcall_info *fci, zend_fcall_info_cache *fcc, zend_long ttl, zend_long now, zval *return_value) {
 	apc_cache_entry_t *entry = NULL;
-	zend_bool bailout = 0;
 
 	if(!cache || apc_cache_busy(cache)) {
         return;
@@ -1933,6 +1886,7 @@ PHP_APCU_API void apc_cache_entry(apc_cache_t *cache, zval *key, zend_fcall_info
 		return;
 	}
 
+	php_apc_try({
 #ifndef APC_LOCK_RECURSIVE
 	if (APCG(recursion)++ == 0) {
 		APC_LOCK(cache->header);
@@ -1940,8 +1894,7 @@ PHP_APCU_API void apc_cache_entry(apc_cache_t *cache, zval *key, zend_fcall_info
 #else
 	APC_LOCK(cache->header);
 #endif
-
-	zend_try {
+	}, {
 		entry = apc_cache_find_internal(cache, Z_STR_P(key), now, 0);
 		if (!entry) {
 			int result = 0;
@@ -1962,10 +1915,7 @@ PHP_APCU_API void apc_cache_entry(apc_cache_t *cache, zval *key, zend_fcall_info
 				}
 			}
 		} else apc_cache_fetch_internal(cache, Z_STR_P(key), entry, now, &return_value);
-	} zend_catch {
-		bailout = 1;
-	} zend_end_try();
-
+	}, {
 #ifndef APC_LOCK_RECURSIVE
 	if (--APCG(recursion) == 0) {
 		APC_UNLOCK(cache->header);
@@ -1973,10 +1923,7 @@ PHP_APCU_API void apc_cache_entry(apc_cache_t *cache, zval *key, zend_fcall_info
 #else
 	APC_UNLOCK(cache->header);
 #endif
-
-	if (bailout) {
-		zend_bailout();
-	}
+	});
 }
 
 /*
